@@ -117,13 +117,19 @@ class Vsys(VersionedPanObject):
         "device.LdapServerProfile",
         "device.SyslogServerProfile",
         "device.HttpServerProfile",
+        "device.LogSettingsSystem",
+        "device.LogSettingsConfig",
+        "device.CertificateProfile",
+        "device.SslDecrypt",
         "objects.AddressObject",
         "objects.AddressGroup",
         "objects.ServiceObject",
         "objects.ServiceGroup",
+        "objects.Tag",
         "objects.ApplicationObject",
         "objects.ApplicationGroup",
         "objects.ApplicationFilter",
+        "objects.ApplicationContainer",
         "objects.ScheduleObject",
         "objects.SecurityProfileGroup",
         "objects.CustomUrlCategory",
@@ -221,6 +227,7 @@ class NTPServer(PanObject):
     # TODO: Add authentication
     # TODO: Add PAN-OS pre-7.0 support
 
+    NAME = None
     XPATH = "/ntp-servers/primary-ntp-server"
 
     def __init__(self, *args, **kwargs):
@@ -371,6 +378,118 @@ class SystemSettings(VersionedPanObject):
         self._params = tuple(params)
 
 
+class LogSettingsSystem(VersionedPanObject):
+    """Firewall or Panorama device log settings system
+
+    Note: This is valid for PANS-OS 8.0+.
+
+    Args:
+        name (string): The name
+        filter (string): Valid values are "All logs" (default) or create your own filter
+        description (string): Description
+        send_to_panorama (bool): Send to panorama
+        send_email (list): Send email profile
+        send_snmp (list): Send snmp profile
+        send_syslog (list): Send syslog profile
+        send_http (list): Send http profile
+
+    """
+
+    ROOT = Root.VSYS
+    SUFFIX = ENTRY
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/log-settings/system/match-list")
+
+        # params
+        params = []
+
+        params.append(VersionedParamPath("filter", default="All Logs", path="filter"))
+
+        params.append(VersionedParamPath("description", path="description"))
+
+        params.append(
+            VersionedParamPath(
+                "send_to_panorama", vartype="yesno", path="send-to-panorama"
+            )
+        )
+
+        params.append(
+            VersionedParamPath("send_email", vartype="member", path="send-email")
+        )
+
+        params.append(
+            VersionedParamPath("send_snmp", vartype="member", path="send-snmptrap")
+        )
+
+        params.append(
+            VersionedParamPath("send_syslog", vartype="member", path="send-syslog")
+        )
+
+        params.append(
+            VersionedParamPath("send_http", vartype="member", path="send-http")
+        )
+
+        self._params = tuple(params)
+
+
+class LogSettingsConfig(VersionedPanObject):
+    """Firewall or Panorama device log settings configuration
+
+    Note: This is valid for PANS-OS 8.0+.
+
+    Args:
+        name (string): The name
+        filter (string): Valid values are "All logs" (default) or create your own filter
+        description (string): Description
+        send_to_panorama (bool): Send to panorama
+        send_email (list): Send email profile
+        send_snmp (list): Send snmp profile
+        send_syslog (list): Send syslog profile
+        send_http (list): Send http profile
+
+    """
+
+    ROOT = Root.VSYS
+    SUFFIX = ENTRY
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/log-settings/config/match-list")
+
+        # params
+        params = []
+
+        params.append(VersionedParamPath("filter", default="All Logs", path="filter"))
+
+        params.append(VersionedParamPath("description", path="description"))
+
+        params.append(
+            VersionedParamPath(
+                "send_to_panorama", vartype="yesno", path="send-to-panorama"
+            )
+        )
+
+        params.append(
+            VersionedParamPath("send_email", vartype="member", path="send-email")
+        )
+
+        params.append(
+            VersionedParamPath("send_snmp", vartype="member", path="send-snmptrap")
+        )
+
+        params.append(
+            VersionedParamPath("send_syslog", vartype="member", path="send-syslog")
+        )
+
+        params.append(
+            VersionedParamPath("send_http", vartype="member", path="send-http")
+        )
+
+        self._params = tuple(params)
+
+
 class PasswordProfile(VersionedPanObject):
     """Password profile object
 
@@ -689,6 +808,8 @@ class Administrator(VersionedPanObject):
         role_profile (str): The role based profile
         password_hash (encrypted str): The encrypted password
         password_profile (str): The password profile for this user
+        vsys_device (list): The vsys list (excluded)
+        vsys_read_only_device (list): The read-only device list (excluded)
 
     """
 
@@ -1032,6 +1153,14 @@ class EmailServer(VersionedPanObject):
         to (str): To email address
         also_to (str): Additional destination email address
         email_gateway (str): IP address or FQDN of email gateway to use
+        protocol (str): (PAN-OS 10.0+) SMTP for clear-text or TLS for encrypted
+        port (int): (PAN-OS 10.0+) Port number
+        tls_version (str): (PAN-OS 10.0+) TLS handshake protocol version.
+        auth (str): (PAN-OS 10.0+) Authentication type.
+        certificate_profile (str): (PAN-OS 10.0+) Certificate profile for
+            validating server certificate.
+        username (str): (PAN-OS 10.0+) Authentication username.
+        password (str): (PAN-OS 10.0+) Authentication password.
 
     """
 
@@ -1050,6 +1179,22 @@ class EmailServer(VersionedPanObject):
         params.append(VersionedParamPath("to", path="to"))
         params.append(VersionedParamPath("also_to", path="and-also-to"))
         params.append(VersionedParamPath("email_gateway", path="gateway"))
+        params.append(VersionedParamPath("protocol", exclude=True, default="SMTP"))
+        params[-1].add_profile("10.0.0", path="protocol", values=["SMTP", "TLS"])
+        params.append(VersionedParamPath("port", default=25, exclude=True))
+        params[-1].add_profile("10.0.0", path="port", vartype="int")
+        params.append(VersionedParamPath("tls_version", exclude=True))
+        params[-1].add_profile("10.0.0", path="tls-version", values=("1.2", "1.1"))
+        params.append(VersionedParamPath("auth", exclude=True))
+        params[-1].add_profile("10.0.0", path="auth", values=("Auto", "Login", "Plain"))
+        params.append(
+            VersionedParamPath("certificate_profile", default="None", exclude=True)
+        )
+        params[-1].add_profile("10.0.0", path="certificate-profile")
+        params.append(VersionedParamPath("username", exclude=True))
+        params[-1].add_profile("10.0.0", path="username")
+        params.append(VersionedParamPath("password", exclude=True))
+        params[-1].add_profile("10.0.0", path="password", vartype="encrypted")
 
         self._params = tuple(params)
 
@@ -1947,3 +2092,242 @@ class HttpIpTagParam(ValueEntry):
 
     LOCATION = "/format/iptag/params"
     ROOT = Root.PANORAMA_VSYS
+
+
+class CertificateProfile(VersionedPanObject):
+    """Certificate profile object.
+
+    Args:
+        name (str): The name
+        username_field (str): The username field.  Valid values are "subject",
+            "subject-alt", or "none".
+        username_field_value (str): The value for the given `username_field`.
+        domain (str): The domain.
+        use_crl (bool): Use CRL.
+        use_ocsp (bool): Use OCSP.
+        crl_receive_timeout (int): CRL receive timeout (sec).
+        ocsp_receive_timeout (int): OCSP receive timeout (sec).
+        certificate_status_timeout (int): Certificate status timeout (sec).
+        block_unknown_certificate (bool): Block session if certificate status
+            is unknown.
+        block_certificate_timeout (bool): Block if a session certificate status
+            can't be retrieved within timeout.
+        block_unauthenticated_certificate (bool): (PAN-OS 7.1) Block session if the
+            certificate was not issued to the authenticating device.
+        block_expired_certificate (bool): (PAN-OS 8.1) Block session if the certificate
+            is expired.
+        ocsp_exclude_nonce (bool): (PAN-OS 9.0) Whether to exclude nonce extension for
+            OCSP requests.
+    """
+
+    # TODO(shinmog): So, in the Panorama debug it seems like /config/shared is
+    # also a valid XPATH, but not sure how to structure the object tree to allow
+    # users to choose between /config/panorama and /config/shared right now.
+    ROOT = Root.PANORAMA_VSYS
+    SUFFIX = ENTRY
+    CHILDTYPES = ("device.CertificateProfileCaCertificate",)
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/certificate-profile")
+
+        # params
+        params = []
+
+        params.append(
+            VersionedParamPath(
+                "username_field",
+                path="username-field/{username_field}",
+                values=("subject", "subject-alt"),
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "username_field_value", path="username-field/{username_field}",
+            )
+        )
+        params.append(VersionedParamPath("domain", path="domain",))
+        params.append(VersionedParamPath("use_crl", path="use-crl",))
+        params.append(VersionedParamPath("use_ocsp", path="use-ocsp",))
+        params.append(
+            VersionedParamPath(
+                "crl_receive_timeout",
+                default=5,
+                vartype="int",
+                path="crl-receive-timeout",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "ocsp_receive_timeout",
+                default=5,
+                vartype="int",
+                path="ocsp-receive-timeout",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "certificate_status_timeout",
+                default=5,
+                vartype="int",
+                path="cert-status-timeout",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "block_unknown_certificate", vartype="yesno", path="block-unknown-cert",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "block_certificate_timeout", vartype="yesno", path="block-timeout-cert",
+            )
+        )
+        params.append(
+            VersionedParamPath("block_unauthenticated_certificate", exclude=True,)
+        )
+        params[-1].add_profile(
+            "7.1.0", vartype="yesno", path="block-unauthenticated-cert",
+        )
+        params.append(VersionedParamPath("block_expired_certificate", exclude=True,))
+        params[-1].add_profile(
+            "8.1.0", vartype="yesno", path="block-expired-cert",
+        )
+        params.append(VersionedParamPath("ocsp_exclude_nonce", exclude=True,))
+        params[-1].add_profile(
+            "9.0.0", path="ocsp-exclude-nonce", vartype="yesno",
+        )
+
+        self._params = tuple(params)
+
+
+class CertificateProfileCaCertificate(VersionedPanObject):
+    """CA certificate for a certificate profile.
+
+    Args:
+        name (str): The name.
+        default_ocsp_url (str): Default URL for OCSP verification.
+        ocsp_verify_certificate (str): Certificate to verify signature in OCSP response.
+        template_name (str): (PAN-OS 9.0+) Template name / OID for the certificate.
+
+    """
+
+    ROOT = Root.PANORAMA_VSYS
+    SUFFIX = ENTRY
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/CA")
+
+        # params
+        params = []
+
+        params.append(VersionedParamPath("default_ocsp_url", path="default-ocsp-url",))
+        params.append(
+            VersionedParamPath("ocsp_verify_certificate", path="ocsp-verify-cert",)
+        )
+        params.append(VersionedParamPath("template_name", exclude=True,))
+        params[-1].add_profile(
+            "9.0.0", path="template-name",
+        )
+
+        self._params = tuple(params)
+
+
+class SslDecrypt(VersionedPanObject):
+    """SSL decrypt configuration for certificates.
+
+    Note: PAN-OS 8.0+
+
+    Args:
+        forward_trust_certificate_rsa (str): RSA CA certificate for trusted sites.
+        forward_trust_certificate_ecdsa (str): ECDSA CA certificate for trusted sites.
+        forward_untrust_certificate_rsa (str): RSA CA certificate for untrusted sites.
+        forward_untrust_certificate_ecdsa (str): ECDSA CA certificate for untrusted sites.
+        root_ca_excludes (list): List of predefined root CAs to not trust.
+        trusted_root_cas (list): List of trusted root CAs.
+        disabled_predefined_exclude_certificates (list): Disabled predefined SSL exclude
+            certificates.
+    """
+
+    NAME = None
+    ROOT = Root.VSYS
+    CHILDTYPES = ("device.SslDecryptExcludeCert",)
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/ssl-decrypt")
+
+        # params
+        params = []
+
+        params.append(
+            VersionedParamPath(
+                "forward_trust_certificate_rsa", path="forward-trust-certificate/rsa",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "forward_trust_certificate_ecdsa",
+                path="forward-trust-certificate/ecdsa",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "forward_untrust_certificate_rsa",
+                path="forward-untrust-certificate/rsa",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "forward_untrust_certificate_ecdsa",
+                path="forward-untrust-certificate/ecdsa",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "root_ca_excludes", vartype="member", path="root-ca-exclude-list",
+            )
+        )
+        # Only option present on Panorama.
+        params.append(
+            VersionedParamPath(
+                "trusted_root_cas", vartype="member", path="trusted-root-CA",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "disabled_predefined_exclude_certificates",
+                vartype="member",
+                path="disabled-ssl-exclude-cert-from-predefined",
+            )
+        )
+
+        self._params = tuple(params)
+
+
+class SslDecryptExcludeCert(VersionedPanObject):
+    """SSL decryption exclusion object.
+
+    Note: PAN-OS 8.0+
+
+    Args:
+        name (str): The name.
+        description (str): Description.
+        exclude (bool): Exclude boolean.
+    """
+
+    ROOT = Root.VSYS
+    SUFFIX = ENTRY
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/ssl-exclude-cert")
+
+        # params
+        params = []
+
+        params.append(VersionedParamPath("description", path="description",))
+        params.append(VersionedParamPath("exclude", vartype="yesno", path="exclude",))
+
+        self._params = tuple(params)
