@@ -72,6 +72,8 @@ class Firewall(PanDevice):
         "device.LdapServerProfile",
         "device.SyslogServerProfile",
         "device.HttpServerProfile",
+        "device.CertificateProfile",
+        "device.SslDecrypt",
         "ha.HighAvailability",
         "objects.AddressObject",
         "objects.AddressGroup",
