@@ -123,7 +123,7 @@ class DeviceGroupHierarchy(object):
 
         dev = self.obj.panorama()
         state = dev.opstate.dg_hierarchy.fetch()
-        self.parent = state[self.obj.uid]
+        self.parent = state.get(self.obj.uid)
 
     def update(self):
         """Change this device group's hierarchical parent.
@@ -181,6 +181,7 @@ class Template(VersionedPanObject):
         "device.LogSettingsConfig",
         "device.PasswordProfile",
         "device.Administrator",
+        "device.SslDecrypt",
         "ha.HighAvailability",
         "network.EthernetInterface",
         "network.AggregateInterface",
@@ -389,6 +390,8 @@ class Panorama(base.PanDevice):
         "device.LdapServerProfile",
         "device.SyslogServerProfile",
         "device.HttpServerProfile",
+        "device.CertificateProfile",
+        "device.SslDecrypt",
         "firewall.Firewall",
         "panorama.DeviceGroup",
         "panorama.Template",
